import numpy as np

# Función para verificar requisitos de Perron-Frobenius y calcular el autovalor y autovector de Perron
def perron_frobenius(matrix):
    # Verificar si la matriz es cuadrada
    if matrix.shape[0] != matrix.shape[1]:
        raise ValueError("La matriz no es cuadrada.")

    # Verificar si la matriz es no negativa
    if np.any(matrix < 0):
        raise ValueError("La matriz contiene elementos negativos.")

    # Verificar si la matriz es primitiva, y por tanto irreducible
    if not np.all(np.linalg.matrix_power(matrix, matrix.shape[0]) > 0):
        raise ValueError("La matriz no es primitiva.")

    # Calcular los autovalores y autovectores asociados
    eigenvalues, eigenvectors = np.linalg.eig(matrix)
    print("\nAutovalores y Autovectores asociados:")

    # Recorre la lista de autovalores y autovectores
    for i in range(len(eigenvalues)):
        eigenvalue = eigenvalues[i]
        eigenvector = eigenvectors[:, i]  # Los autovectores están almacenados como columnas

        # Imprime el autovalor y el autovector asociado
        print(f"Autovalor {i + 1}: {eigenvalue}")
        print(f"Autovector {i + 1}: {eigenvector}")
        print()



    # Obtener el índice del valor propio de Perron (máximo valor propio)
    perron_index = np.argmax(np.real(eigenvalues))
    # Obtener el autovector correspondiente al valor propio de Perron
    perron_vector = np.real(eigenvectors[:, perron_index])

    # Normalizar el autovector para que la suma de sus componentes sea 1
    perron_vector /= np.sum(perron_vector)
    # Redondear el resultado a 4 decimales
    perron_vector = np.round(perron_vector, decimals=4)

    return perron_vector, np.real(eigenvalues[perron_index])


# Matriz de transición del ejemplo de la NBA
user_matrix = np.array([[0, 6/14, 0/14, 3/14],
                        [0/14, 0, 2/14, 2/14],
                        [4/14, 2/14, 0, 2/14],
                        [1/14, 2/14, 4/14, 0]])

print("Matriz A:")
print(user_matrix)

# Pregunta al usuario si desea trasponer la matriz
respuesta = input("¿Desea trasponer la matriz? (S/N): ")

# Trasponer la matriz si la respuesta es 'S' o 's'
if respuesta.upper() == 'S':
    user_matrix = user_matrix.T

# Verificar el Teorema de Perron-Frobenius y calcular el vector de Perron
perron_vector, perron_value = perron_frobenius(user_matrix)

print("\nVector propio de Perron:")
print(perron_vector)
print("\nValor propio de Perron:")
print(perron_value)